@@ -19,17 +19,15 @@
  *   with this program; if not, write to the Free Software Foundation, Inc.,
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  *
- *   $Id: fuseext2.c,v 1.9 2007/05/03 12:19:14 rd235 Exp $
+ *   $Id: fuseext2.c 370 2007-08-03 17:43:34Z rd235 $
  *
  */
 
 //TODO:
-//symlink
 //fsync!?
 //extended attributes
 //flush is correct?!
 //writepage?!
-//file (in functions open) is always writable, is correct?
 //bug ls, sometimes there aren't all file and/or directory
 //
 #if 0
@@ -118,14 +116,11 @@ static int ext2_getattr(const char *path, struct stat *stbuf)
 	return 0;
 }
 
-//TODO error to symlink
 static int ext2_readlink(const char *path, char *buf, size_t size)
 {
 	ext2_ino_t ino_n;
 	struct ext2_inode  ino;
 	int err;
-	char *buffer = 0;
-	char *pathname;
 
 	ext2_filsys e2fs;
 	struct fuse_context *mycontext=fuse_get_context();
@@ -144,7 +139,12 @@ static int ext2_readlink(const char *path, char *buf, size_t size)
 	
 	if (!LINUX_S_ISLNK(ino.i_mode))
 		return -EINVAL;
+	size--;
+	if (ino.i_size < size)
+		size = ino.i_size;
 	if ( ext2fs_inode_data_blocks(e2fs, &ino) ) {
+		/* slow symlink */
+		char *buffer;
 		err = ext2fs_get_mem(e2fs->blocksize, &buffer);
 		if (err)
 			return err;
@@ -153,17 +153,13 @@ static int ext2_readlink(const char *path, char *buf, size_t size)
 			ext2fs_free_mem(&buffer);
 			return err;
 		}
-		pathname = buffer;
+		memcpy(buf,buffer,size);
+		ext2fs_free_mem(&buffer);
 	} else
-		pathname = (char *)&(ino.i_block[0]);
-	size--;
-	if (ino.i_size < size)
-		size = ino.i_size;
-	memcpy(buf,pathname,size);
+		/* fast symlink */
+		memcpy(buf,(char *)&(ino.i_block[0]),size);
 	buf[size]=0;
 
-	if (buffer)
-		ext2fs_free_mem(&buffer);
 	return 0;
 }
 
@@ -318,8 +314,7 @@ static int ext2_open(const char *path, struct fuse_file_info *fi)
 	#endif
 	if(err || ino_n == 0)
 		return -ENOENT;
-	//err=ext2fs_file_open(e2fs, ino_n,  0, &e2file);
-	if (fi->flags & O_ACCMODE != 0)
+	if ((fi->flags & O_ACCMODE) != 0)
 		err = ext2fs_file_open(e2fs, ino_n, EXT2_FILE_WRITE, &e2file);
 	else
 		err = ext2fs_file_open(e2fs, ino_n, 0, &e2file);
@@ -355,7 +350,6 @@ static int ext2_mknod(const char *path, mode_t mode, dev_t dev)
 		fprintf(stderr, "The file '%s' already exists\n", path);
 		return 1;
 	}*/
-	path_parent = strdup(path);
 	#ifdef DEBUG
 	printf("\text2_mknod:%s\n",path);
 	#endif
@@ -364,16 +358,18 @@ static int ext2_mknod(const char *path, mode_t mode, dev_t dev)
 		#ifdef DEBUG
 		fprintf(stderr, "Error to allocate inode:%d\n",retval);
 		#endif
-		return -retval;
+		return -ENOSPC;
 	}
 	#ifdef DEBUG
 	printf("\t\tAllocated inode: %u\n", newfile);
 	#endif
 	
+	if (strlen(path) == 0)
+		path_parent = strdup("/");
+	else
+		path_parent = strdup(path);
 	cp = strrchr(path_parent, '/');
 	*cp = 0;
-	if (strlen(path_parent) == 0)
-		path_parent = "/";
 	cp++;//cp point to file name
 	/*name = strdup(cp); // XXX ERR: this DUP has no FREE!!!*/
 	name = cp;
@@ -385,15 +381,19 @@ static int ext2_mknod(const char *path, mode_t mode, dev_t dev)
 	retval = ext2fs_link(e2fs, parent, name, newfile, EXT2_FT_REG_FILE);
 
 	while (retval == EXT2_ET_DIR_NO_SPACE) {
-		printf("EXT2_ET_DIR_NO_SPACE\n");
+		#ifdef DEBUG
+		fprintf(stderr, "Expand dir space\n");
+		#endif
 		retval = ext2fs_expand_dir(e2fs, parent);
 		if (retval) {
 			fprintf(stderr, "while expanding directory\n");
-			return retval;
+			free(path_parent);
+			return -ENOSPC;
 		}
 		retval = ext2fs_link(e2fs, parent, name, newfile, EXT2_FT_REG_FILE);
 	}
-        if (ext2fs_test_inode_bitmap(e2fs->inode_map, newfile))
+	free(path_parent);
+	if (ext2fs_test_inode_bitmap(e2fs->inode_map, newfile))
 		fprintf(stderr, "Warning: inode already set\n");
 			
 	ext2fs_inode_alloc_stats2(e2fs, newfile, +1, 0);
@@ -408,7 +408,7 @@ static int ext2_mknod(const char *path, mode_t mode, dev_t dev)
 	retval = ext2fs_write_new_inode(e2fs, newfile, &inode);
 	if (retval) {
 		fprintf(stderr, "Error while creating inode %u\n", newfile);
-		return retval;
+		return -EIO;
 	}
 	return 0;
 }
@@ -493,7 +493,7 @@ static int ext2_write(const char *path, const char *buf, size_t size, off_t offs
 		size -= got;
 		buftmp += got;
 	#ifdef DEBUG
-	printf("\t\tCycle:%d\n",size);
+	printf("\t\tCycle:%d %d\n",size,retval);
 	#endif
 		}	
 	#ifdef DEBUG
@@ -545,7 +545,7 @@ static int ext2_mkdir(const char *path, mode_t mode)
 	ext2_ino_t parent;// = NULL;
 	char	*name;
 	char	*cp;
-	char *path_parent = strdup(path);
+	char *path_parent;
 	ext2_ino_t ino_n;
 	struct ext2_inode  ino;
 
@@ -556,17 +556,21 @@ static int ext2_mkdir(const char *path, mode_t mode)
 	printf("\text2_mkdir:%s\n",path_parent);
 	#endif
 	
+	if (strlen(path) == 0)
+		path_parent=strdup("/");
+	else
+		path_parent= strdup(path);
  	cp = strrchr(path_parent, '/');//point to last /
 	if (cp) { 
 		name = strdup(cp+1);
 		*cp = 0;
-		if(!strlen(path_parent))
-			strcat(path_parent, "/");
 		err = ext2fs_namei(e2fs, EXT2_ROOT_INO, EXT2_ROOT_INO, path_parent, &parent);
 		if (!parent) {
 			#ifdef DEBUG
 			printf("Parent Inode not found\n");
 			#endif
+			free(path_parent);
+			free(name);
 			return -ENOENT;
 		}
 
@@ -574,6 +578,8 @@ static int ext2_mkdir(const char *path, mode_t mode)
 		#ifdef DEBUG
 		printf("path without /\n");
 		#endif
+		free(path_parent);
+		free(name);
 		return -ENOENT;
 	}
 	#ifdef DEBUG
@@ -591,6 +597,8 @@ static int ext2_mkdir(const char *path, mode_t mode)
 		err = ext2fs_expand_dir(e2fs, parent);
 		if (err) {
 			fprintf(stderr, "Error while expanding directory\n");
+			free(path_parent);
+			free(name);
 			return -ENOENT;
 		}
 #ifdef DEBUG
@@ -601,6 +609,8 @@ static int ext2_mkdir(const char *path, mode_t mode)
 #ifdef DEBUG
 		printf("\t\tMkdirError:%d\n",err);
 #endif
+		free(path_parent);
+		free(name);
 	}
 	if (err) {
 		fprintf(stderr, "Mkdir error:%d\n",err);
@@ -646,7 +656,7 @@ static int rmdir_proc(ext2_ino_t dir EXT2FS_ATTR((unused)),
 	return 0;
 }
 
-static int unlink_file_by_name(const char *filename)
+static int unlink_file_by_name(ext2_filsys e2fs,const char *filename)
 {
 	int		retval;
 	ext2_ino_t	dir;
@@ -655,10 +665,6 @@ static int unlink_file_by_name(const char *filename)
 	if (!localfn)
 		return -ENOMEM;
 	
-	ext2_filsys e2fs;
-	struct fuse_context *mycontext=fuse_get_context();
-	e2fs = (ext2_filsys) mycontext->private_data;
-
 	basename = strrchr(localfn, '/');
 	if (basename) {
 		*basename++ = '\0';
@@ -689,35 +695,36 @@ static int release_blocks_proc(ext2_filsys fs, blk_t *blocknr,
 	return 0;
 }
 
-static int kill_file_by_inode(ext2_ino_t inode)
+static int kill_file_by_inode(ext2_filsys e2fs, ext2_ino_t inode,int nlink)
 {
 	struct ext2_inode inode_buf;
 	int retval;
 	
-	ext2_filsys e2fs;
-	struct fuse_context *mycontext=fuse_get_context();
-	e2fs = (ext2_filsys) mycontext->private_data;
-
 	retval = ext2fs_read_inode(e2fs, inode, &inode_buf);
 	if(retval)
 		return retval;
-	inode_buf.i_dtime = time(NULL);
-	
+  inode_buf.i_links_count -= nlink;
+	if (inode_buf.i_links_count <= 0) {
+		inode_buf.i_links_count =0;
+		inode_buf.i_dtime = time(NULL);
+	}
 	retval = ext2fs_write_inode(e2fs, inode, &inode_buf);
 	if(retval)
 		return retval;
 
-	if (!ext2fs_inode_has_valid_blocks(&inode_buf))
-		return 1;
+	if (inode_buf.i_links_count == 0) {
 
-	ext2fs_block_iterate(e2fs, inode, 0, NULL,
-			     release_blocks_proc, NULL);
-	ext2fs_inode_alloc_stats2(e2fs, inode, -1,
-				  LINUX_S_ISDIR(inode_buf.i_mode));
+		if (ext2fs_inode_has_valid_blocks(&inode_buf)) {
+			ext2fs_block_iterate(e2fs, inode, 0, NULL,
+					release_blocks_proc, NULL);
+		}
+		ext2fs_inode_alloc_stats2(e2fs, inode, -1,
+				LINUX_S_ISDIR(inode_buf.i_mode));
+	}
 	return 0;
 }
 
-static int ext2_rmdir(const char *path)
+	static int ext2_rmdir(const char *path)
 {
 
 	int retval;
@@ -734,12 +741,11 @@ static int ext2_rmdir(const char *path)
 	printf("\text2_rmdir\n\t\tpath:%s\n\t\tnameiErr:%d\n",path, retval);
 	#endif
 	if (retval) {
-		fprintf(stderr, "while trying to resolve filename");
+		fprintf(stderr, "while trying to resolve filename %s\n",path);
 		return -ENOENT;
 	}
 	
 	retval = ext2fs_read_inode(e2fs, inode_num, &inode);
-	printf("\t\text2fs_read_inodeERR:%d\n",retval);
 
 	if (!LINUX_S_ISDIR(inode.i_mode)) {
 		printf("ERROR,file is not a directory\n");
@@ -763,25 +769,15 @@ static int ext2_rmdir(const char *path)
 		return -ENOTEMPTY;
 	}
 
-	inode.i_links_count = 0;
-	retval = ext2fs_write_inode(e2fs, inode_num, &inode);
-	#ifdef DEBUG
-	printf("\t\text2fs_write_inodeERR:%d\n",retval);
-	#endif
-		
-	if (retval) {
-		printf("while writing inode %u", inode_num);
-		return 1;
-	}
-	unlink_file_by_name(path);
-	kill_file_by_inode(inode_num);
+	unlink_file_by_name(e2fs,path);
+	kill_file_by_inode(e2fs,inode_num,2);
 	if (rds.parent) {
 		if ( retval = ext2fs_read_inode(e2fs, rds.parent, &inode) )
-			return retval;
+			return -EIO;
 		if (inode.i_links_count > 1)
 			inode.i_links_count--;
 		if ( retval = ext2fs_write_inode(e2fs, rds.parent, &inode) )
-			return retval;
+			return -EIO;
 	}
 	return 0;	
 }
@@ -823,7 +819,6 @@ static int ext2_symlink(const char *sourcename, const char *destname)
 	const char	*dest;
 	char  *cp, *basename;
 
-
 	ext2_filsys e2fs;
 	struct fuse_context *mycontext=fuse_get_context();
 	e2fs = (ext2_filsys) mycontext->private_data;
@@ -850,14 +845,69 @@ static int ext2_symlink(const char *sourcename, const char *destname)
 		}
 	}
 
-	//retval = ext2fs_link(e2fs, dir, dest, ino, ext2_file_type(inode.i_mode));
+	retval = ext2fs_new_inode(e2fs, EXT2_ROOT_INO, 010755, 0, &ino);
+	if (retval) {
+#ifdef DEBUG
+		fprintf(stderr, "Error to allocate inode:%d\n",retval);
+#endif
+		return -ENOSPC;
+	}
+  
+	retval = ext2fs_link(e2fs, dir, dest, ino, EXT2_FT_SYMLINK);
+	while (retval == EXT2_ET_DIR_NO_SPACE) {
+#ifdef DEBUG
+		fprintf(stderr, "Expand dir space\n");
+#endif
+		retval = ext2fs_expand_dir(e2fs, dir);
+		if (retval) {
+			fprintf(stderr, "while expanding directory\n");
+			return -ENOSPC;
+		}
+		retval = ext2fs_link(e2fs, dir, dest, ino, EXT2_FT_SYMLINK);
+	}
+
+	if (ext2fs_test_inode_bitmap(e2fs->inode_map, ino))
+		fprintf(stderr, "Warning: inode already set\n");
+
+	ext2fs_inode_alloc_stats2(e2fs, ino, +1, 0);
+	memset(&inode, 0, sizeof(inode));
+	inode.i_mode = S_IFLNK | 0777;
+	inode.i_atime = inode.i_ctime = inode.i_mtime = time(NULL);
+	inode.i_links_count = 1;
+	inode.i_size = strlen(sourcename); //initial size of file
+	inode.i_uid=mycontext->uid;
+	inode.i_gid=mycontext->gid;
+
+	if (strlen(sourcename) <= sizeof(inode.i_block)) {
+		/* fast symlink */
+		strncpy((char *)&(inode.i_block[0]),sourcename,sizeof(inode.i_blocks));
+	} else {
+		/* slow symlink */
+		char *buffer;
+		int err;
+		err = ext2fs_get_mem(e2fs->blocksize, &buffer);
+		if (!err) {
+			blk_t blk;
+			strncpy(buffer,sourcename,e2fs->blocksize);
+			err = ext2fs_new_block(e2fs, 0, 0, &blk);
+			if (!err) {
+				inode.i_block[0]=blk;
+				inode.i_blocks=e2fs->blocksize / 512;
+				err = io_channel_write_blk(e2fs->io, blk, 1, buffer);
+				ext2fs_block_alloc_stats(e2fs, blk, +1);
+			}
+			ext2fs_free_mem(&buffer);
+		}
+	}
+
+	retval = ext2fs_write_new_inode(e2fs, ino, &inode);
+
 	#ifdef DEBUG
-	fprintf(stderr, "\t\text2fs_link error:%d\n",retval);
+	fprintf(stderr, "\t\text2fs_symlink error:%d\n",retval);
 	#endif
 	return 0;
 }
 
-
 static int ext2_link(const char *sourcename, const char *destname)
 {
 	ext2_ino_t	ino;
@@ -906,9 +956,23 @@ static int ext2_link(const char *sourcename, const char *destname)
 	retval = ext2fs_read_inode(e2fs, ino, &inode);
 	if (retval) {
 		fprintf(stderr, "while reading inode %u", ino);
-		return 1;
+		return -EIO;
 	}
 	retval = ext2fs_link(e2fs, dir, dest, ino, ext2_file_type(inode.i_mode));
+	while (retval == EXT2_ET_DIR_NO_SPACE) {
+		#ifdef DEBUG
+		fprintf(stderr, "Expand dir space\n");
+		#endif
+		retval = ext2fs_expand_dir(e2fs, dir);
+		if (retval) {
+			fprintf(stderr, "while expanding directory\n");
+			return -ENOSPC;
+		}
+		retval = ext2fs_link(e2fs, dir, dest, ino, ext2_file_type(inode.i_mode));
+	}
+	if (retval == EXT2_ET_NO_DIRECTORY)
+		return -EEXIST;
+
 	if (retval == 0) {
 		inode.i_links_count ++;
 		retval =  ext2fs_write_inode(e2fs, ino, &inode);
@@ -933,7 +997,7 @@ static int ext2_unlink(const char *path)
 
 	retval = ext2fs_namei(e2fs, EXT2_ROOT_INO, EXT2_ROOT_INO, path, &inode_num);
 	if (retval) {
-		printf("while trying to resolve filename");
+		printf("while trying to resolve filename %s\n",path);
 		return -ENOENT;
 	}
 	
@@ -948,16 +1012,9 @@ static int ext2_unlink(const char *path)
 		return -EISDIR;
 	}
 
-	--inode.i_links_count;
-	retval = ext2fs_write_inode(e2fs, inode_num, &inode);
-	if (retval) {
-		fprintf(stderr, "while writing inode %u", inode_num);
-		return -EIO;
-	}
-
-	unlink_file_by_name(path);
-	if (inode.i_links_count == 0)
-		kill_file_by_inode(inode_num);
+	unlink_file_by_name(e2fs,path);
+	//if (inode.i_links_count == 0)
+	kill_file_by_inode(e2fs,inode_num,1);
 
 	return 0;
 }
@@ -1076,7 +1133,103 @@ static int ext2_truncate(const char *path, off_t length)
 
 static int ext2_rename(const char *oldpath, const char *newpath)
 {
-	printf("TODO RENAME\n");
+	ext2_ino_t ino_old;
+	ext2_ino_t ino_new=0;
+	struct ext2_inode inode;
+	int err;
+	int retval;
+	ext2_ino_t dir;
+	char *cp = strrchr(newpath, '/');
+	const char *dest;
+
+	ext2_filsys e2fs;
+	struct fuse_context *mycontext = fuse_get_context();
+	e2fs = (ext2_filsys) mycontext->private_data;
+
+	/*Get the source inode*/
+	retval = ext2fs_namei(e2fs, EXT2_ROOT_INO, EXT2_ROOT_INO, oldpath, &ino_old);
+	if (retval != 0 || ino_old == 0)
+		return -ENOENT; //inode with this name does not exist
+	retval = ext2fs_read_inode(e2fs, ino_old, &inode);
+	if (retval) {
+		printf("while reading inode %u", ino_old);
+		return 1;
+	}
+
+	retval = ext2fs_namei(e2fs, EXT2_ROOT_INO, EXT2_ROOT_INO, newpath, &ino_new);
+
+	if (cp) {
+		*cp = 0;
+		retval = ext2fs_namei(e2fs, EXT2_ROOT_INO, EXT2_ROOT_INO, newpath, &dir);
+		if (retval != 0 || dir == 0)
+			return -ENOENT;
+		dest = cp+1;
+	} else {
+		dir = 2;// XXX ERR cwd;
+		dest = newpath;
+	}
+
+	/*Get the destination inode*/
+	if (ino_new != 0) {
+		if (ino_old == ino_new) /*are the same: DO NOTHING*/
+			return 0;
+		else {
+			struct ext2_inode newinode;
+			/*newpath exists*/
+#ifdef DEBUG
+			fprintf(stderr, "rename existing new\n");
+#endif
+			retval = ext2fs_read_inode(e2fs, ino_new, &newinode);
+			if (retval) {
+				printf("while reading inode %u", ino_old);
+				return 1;
+			}
+
+			if (LINUX_S_ISDIR(newinode.i_mode)) {
+				/* oldpath can specify a directory.  In this case, newpath must either not
+				 *        exist, or it must specify an empty directory. */
+				if (!LINUX_S_ISDIR(inode.i_mode)) {
+					return -EISDIR;
+				} else {
+					struct rd_struct rds;
+					rds.parent = 0;
+					rds.empty = 1;
+					retval = ext2fs_dir_iterate2(e2fs, ino_new, 0, 0, rmdir_proc, &rds);
+					if (retval) {
+						printf("while iterating over directory\n");
+						return -ENOENT;
+					}
+					if (rds.empty == 0) {
+						printf("directory not empty\n");
+						return -ENOTEMPTY;
+					}
+				}
+			}
+
+			retval = ext2fs_unlink(e2fs, dir, dest, 0, 0);
+			if (retval)
+				printf("unlink_file_by_name:%d\n", retval);
+		}
+	} 
+	/* link the old to the new path */
+	retval = ext2fs_link(e2fs, dir, dest, ino_old, ext2_file_type(inode.i_mode));
+	while (retval == EXT2_ET_DIR_NO_SPACE) {
+#ifdef DEBUG
+		fprintf(stderr, "Expand dir space\n");
+#endif
+		retval = ext2fs_expand_dir(e2fs, dir);
+		if (retval == 0) 
+			retval = ext2fs_link(e2fs, dir, dest, ino_old, ext2_file_type(inode.i_mode));
+	}
+
+	if (retval == 0) {
+		/* unlink the old path */
+		unlink_file_by_name(e2fs,oldpath);
+		if (ino_new != 0) 
+			kill_file_by_inode(e2fs,ino_new,1);
+	} else {
+		retval=ext2fs_link(e2fs, dir, dest, ino_new, ext2_file_type(inode.i_mode));
+	}
 	return 0;
 }
 
@@ -1158,7 +1311,7 @@ static int ext2_flush(const char *path, struct fuse_file_info *fi)
 	#endif
 
 //	if ((e2file->flags & EXT2_FILE_BUF_VALID) || (e2file->flags & EXT2_FILE_BUF_DIRTY))
-//		printf("Ce roba da flusciare!!!!!!!!!!!!!!!\n");
+//		printf("flush needed\n");
 	#ifdef DEBUG
 	printf("\t\tfiletab:%d\n",fi->fh);
 	#endif
@@ -1239,28 +1392,6 @@ static struct fuse_operations ext2_oper = {
 #if ( FUSE_MINOR_VERSION >= 5 )
 	.statfs		= ext2_statfs,
 #endif
-/*
-ok    .getattr	= ext2_getattr,
-ok    .readlink	= ext2_readlink,
-ok    .getdir	= ext2_getdir,
-ok    .mknod	= ext2_mknod,
-ok    .mkdir	= ext2_mkdir,
-process    .symlink	= ext2_symlink,
-ok    .unlink	= ext2_unlink,
-ok    .rmdir	= ext2_rmdir,
-process    .rename	= ext2_rename,
-process    .link	= ext2_link,
-process    .chmod	= ext2_chmod,
-process    .chown	= ext2_chown,
-process     .truncate	= ext2_truncate,
-prcess    .utime	= ext2_utime,
-ok    .open	= ext2_open,
-ok    .read	= ext2_read,
-proces    .write	= ext2_write,
-process    .statfs	= ext2_statfs,
-ok    .release	= ext2_release,
-    .fsync	= ext2_fsync,
-    */
 #if 0
 #ifdef HAVE_SETXATTR
     .setxattr	= ext2_setxattr,
@@ -1366,33 +1497,6 @@ int main(int argc, char *argv[])
 		}
 	}
 
-#if 0
-#if ( FUSE_MINOR_VERSION <= 4 ) 
-	fuse_fd = fuse_mount(mountpoint, "rw");//vuole il  mountpoint, attenzione rw e' dummy, e' ignorato da umfuse ma non libfuse!!
-#else
-	argc -=2;
-	argv[argc]=0;
-	struct fuse_args args=FUSE_ARGS_INIT(argc, argv);
-	fuse_fd = fuse_mount(mountpoint, &args);
-#endif
-#ifdef DEBUG
-printf("fuse-fd %d %d\n",fuse_fd,FUSE_MINOR_VERSION);
-#endif
-#if ( FUSE_MINOR_VERSION <= 5 )
-	fuse = fuse_new(fuse_fd, NULL, &ext2_oper, sizeof(ext2_oper));
-	init_data=e2fs;
-#else
-	fuse = fuse_new(fuse_fd, &args, &ext2_oper, sizeof(ext2_oper), e2fs);
-#endif
-
-//fuse_main(argc, argv, &ext2_oper);
-#ifdef DEBUG
-	printf("InLoop\n");
-#endif
-	fuse_loop(fuse);
-	//fuse_loop_mt(fuse);
-#endif
-
 	argv[1]=argv[0];
 #if ( FUSE_MINOR_VERSION <= 5 )
 	init_data=e2fs;
